"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NNCrypto = void 0;
const libsodium_wrappers_1 = require("libsodium-wrappers");
const decryption_1 = __importDefault(require("./src/decryption"));
const encryption_1 = __importDefault(require("./src/encryption"));
const keyutils_1 = __importDefault(require("./src/keyutils"));
const password_1 = __importDefault(require("./src/password"));
class NNCrypto {
    constructor() {
        this.isReady = false;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isReady)
                return;
            yield libsodium_wrappers_1.ready;
            this.isReady = true;
        });
    }
    encrypt(key, plaintext, outputFormat = "uint8array") {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return encryption_1.default.encrypt(key, plaintext, outputFormat);
        });
    }
    decrypt(key, cipherData, outputFormat = "text") {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return decryption_1.default.decrypt(key, cipherData, outputFormat);
        });
    }
    hash(password, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return password_1.default.hash(password, salt);
        });
    }
    deriveKey(password, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return keyutils_1.default.deriveKey(password, salt);
        });
    }
    exportKey(password, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return keyutils_1.default.exportKey(password, salt);
        });
    }
    createEncryptionStream(key, stream) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            const encryptionStream = encryption_1.default.createStream(key);
            while (true) {
                const chunk = yield stream.read();
                if (!chunk)
                    break;
                const { data, final } = chunk;
                if (!data)
                    break;
                const encryptedChunk = {
                    data: encryptionStream.write(data, final),
                    final,
                };
                yield stream.write(encryptedChunk);
                if (final)
                    break;
            }
            return encryptionStream.header;
        });
    }
    createDecryptionStream(iv, key, stream) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            const decryptionStream = decryption_1.default.createStream(iv, key);
            while (true) {
                const chunk = yield stream.read();
                if (!chunk)
                    break;
                const { data, final } = chunk;
                if (!data)
                    break;
                const decryptedChunk = {
                    data: decryptionStream.read(data),
                    final,
                };
                yield stream.write(decryptedChunk);
                if (final)
                    break;
            }
        });
    }
    encryptStream(key, stream, _streamId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            return yield this.createEncryptionStream(key, stream);
        });
    }
    decryptStream(key, iv, stream, _streamId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            yield this.createDecryptionStream(iv, key, stream);
        });
    }
}
exports.NNCrypto = NNCrypto;
