"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const libsodium_wrappers_1 = require("libsodium-wrappers");
const keyutils_1 = __importDefault(require("./keyutils"));
class Decryption {
    static transformInput(cipherData) {
        let input = null;
        if (typeof cipherData.cipher === "string" &&
            cipherData.format === "base64") {
            input = (0, libsodium_wrappers_1.from_base64)(cipherData.cipher, libsodium_wrappers_1.base64_variants.URLSAFE_NO_PADDING);
        }
        else if (typeof cipherData.cipher === "string" &&
            cipherData.format === "hex") {
            input = (0, libsodium_wrappers_1.from_hex)(cipherData.cipher);
        }
        else if (cipherData.cipher instanceof Uint8Array) {
            input = cipherData.cipher;
        }
        if (!input)
            throw new Error("Data cannot be null.");
        return input;
    }
    static decrypt(key, cipherData, outputFormat = "text") {
        if (!key.salt && cipherData.salt)
            key.salt = cipherData.salt;
        const encryptionKey = keyutils_1.default.transform(key);
        const input = this.transformInput(cipherData);
        const plaintext = (0, libsodium_wrappers_1.crypto_aead_xchacha20poly1305_ietf_decrypt)(null, input, null, (0, libsodium_wrappers_1.from_base64)(cipherData.iv), encryptionKey.key);
        return {
            format: outputFormat,
            data: outputFormat === "base64"
                ? (0, libsodium_wrappers_1.to_base64)(plaintext, libsodium_wrappers_1.base64_variants.ORIGINAL)
                : outputFormat === "text"
                    ? (0, libsodium_wrappers_1.to_string)(plaintext)
                    : plaintext,
        };
    }
    static createStream(header, key) {
        return new DecryptionStream(header, keyutils_1.default.transform(key));
    }
}
exports.default = Decryption;
class DecryptionStream {
    constructor(header, key) {
        this.state = (0, libsodium_wrappers_1.crypto_secretstream_xchacha20poly1305_init_pull)((0, libsodium_wrappers_1.from_base64)(header), key.key);
    }
    read(chunk) {
        const { message } = (0, libsodium_wrappers_1.crypto_secretstream_xchacha20poly1305_pull)(this.state, chunk, null);
        return message;
    }
}
