"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const libsodium_wrappers_1 = require("libsodium-wrappers");
const keyutils_1 = __importDefault(require("./keyutils"));
const encoder = new TextEncoder();
class Encryption {
    static transformInput(plaintext) {
        let data = null;
        if (typeof plaintext.data === "string" && plaintext.format === "base64") {
            data = (0, libsodium_wrappers_1.from_base64)(plaintext.data, libsodium_wrappers_1.base64_variants.ORIGINAL);
        }
        else if (typeof plaintext.data === "string") {
            data = encoder.encode(plaintext.data);
        }
        else if (plaintext.data instanceof Uint8Array) {
            data = plaintext.data;
        }
        if (!data)
            throw new Error("Data cannot be null.");
        return data;
    }
    static encrypt(key, plaintext, outputFormat = "uint8array") {
        const encryptionKey = keyutils_1.default.transform(key);
        const data = this.transformInput(plaintext);
        const nonce = (0, libsodium_wrappers_1.randombytes_buf)(libsodium_wrappers_1.crypto_aead_xchacha20poly1305_ietf_NPUBBYTES);
        const cipher = (0, libsodium_wrappers_1.crypto_aead_xchacha20poly1305_ietf_encrypt)(data, null, null, nonce, encryptionKey.key);
        let output = cipher;
        if (outputFormat === "base64") {
            output = (0, libsodium_wrappers_1.to_base64)(cipher, libsodium_wrappers_1.base64_variants.URLSAFE_NO_PADDING);
        }
        const iv = (0, libsodium_wrappers_1.to_base64)(nonce);
        return {
            format: outputFormat,
            alg: getAlgorithm(libsodium_wrappers_1.base64_variants.URLSAFE_NO_PADDING),
            cipher: output,
            iv,
            salt: encryptionKey.salt,
            length: data.length,
        };
    }
    static createStream(key) {
        return new EncryptionStream(keyutils_1.default.transform(key));
    }
}
exports.default = Encryption;
class EncryptionStream {
    constructor(key) {
        const { state, header } = (0, libsodium_wrappers_1.crypto_secretstream_xchacha20poly1305_init_push)(key.key, "base64");
        this.state = state;
        this.header = header;
    }
    write(chunk, final) {
        return (0, libsodium_wrappers_1.crypto_secretstream_xchacha20poly1305_push)(this.state, chunk, null, final
            ? libsodium_wrappers_1.crypto_secretstream_xchacha20poly1305_TAG_FINAL
            : libsodium_wrappers_1.crypto_secretstream_xchacha20poly1305_TAG_MESSAGE);
    }
}
function getAlgorithm(base64Variant) {
    //Template: encryptionAlgorithm-kdfAlgorithm-base64variant
    return `xcha-argon2i13-${base64Variant}`;
}
