"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const libsodium_wrappers_1 = require("libsodium-wrappers");
class KeyUtils {
    static deriveKey(password, salt) {
        let saltBytes;
        if (!salt)
            saltBytes = (0, libsodium_wrappers_1.randombytes_buf)(libsodium_wrappers_1.crypto_pwhash_SALTBYTES);
        else {
            saltBytes = (0, libsodium_wrappers_1.from_base64)(salt);
        }
        if (!saltBytes)
            throw new Error("Could not generate bytes from the given salt.");
        const key = (0, libsodium_wrappers_1.crypto_pwhash)(libsodium_wrappers_1.crypto_aead_xchacha20poly1305_ietf_KEYBYTES, password, saltBytes, 3, // operations limit
        1024 * 1024 * 8, // memory limit (8MB)
        libsodium_wrappers_1.crypto_pwhash_ALG_ARGON2I13);
        return {
            key,
            salt: typeof salt === "string" ? salt : (0, libsodium_wrappers_1.to_base64)(saltBytes),
        };
    }
    static exportKey(password, salt) {
        const { key, salt: keySalt } = this.deriveKey(password, salt);
        return { key: (0, libsodium_wrappers_1.to_base64)(key), salt: keySalt };
    }
    /**
     * Takes in either a password or a serialized encryption key
     * and spits out a key that can be directly used for encryption/decryption.
     * @param input
     */
    static transform(input) {
        if ("password" in input && !!input.password) {
            const { password, salt } = input;
            return this.deriveKey(password, salt);
        }
        else if ("key" in input && !!input.salt && !!input.key) {
            return { key: (0, libsodium_wrappers_1.from_base64)(input.key), salt: input.salt };
        }
        throw new Error("Invalid input.");
    }
}
exports.default = KeyUtils;
